/*  GTKtalog.
 *  Copyright (C) 2000  Yves Mettier
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


%{
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include "config_common.h"
#include "interface.h"
#include "io.h"
#include "setup.h"

GList *local_config;

%}

%union {
	gint		val;
	char		*string;
	gboolean        gbool;  
	GString		*gstring;
	GArray		*garray;
	GList		*glist;
	KEY_VALUE	*kv;
}

/* not used yet
%type <glist>		INTEGERS
*/
%type <glist>		STRINGS
%type <gstring>		GSTRING
%type <kv>		one_line
%type <glist>		lines
%type <glist>		config_file

%type <kv>		line_ADDICON
%token 			ADDICON

%type <kv>		line_DEFAULTDIRICON
%token 			DEFAULTDIRICON

%type <kv>		line_DEFAULTDISKICON
%token 			DEFAULTDISKICON

%type <kv>		line_DEFAULTFILEICON
%token 			DEFAULTFILEICON

%type <kv>		line_DEFAULTUNREADABLEICON
%token 			DEFAULTUNREADABLEICON

%type <kv>		line_DEFAULTVFSICON
%token 			DEFAULTVFSICON

%type <kv>		line_MOUNTPROGRAM
%token 			MOUNTPROGRAM

%type <kv>		line_UMOUNTPROGRAM
%token 			UMOUNTPROGRAM

%type <kv>		line_EJECTPROGRAM
%token 			EJECTPROGRAM

%type <kv>		line_MOUNTPOINT
%token 			MOUNTPOINT

%type <kv>		line_USEAUTOMOUNT
%token 			USEAUTOMOUNT

%type <kv>		line_EXTENSION
%token 			EXTENSION

%type <kv>		line_OPTIONDUS
%token 			OPTIONDUS

%type <kv>		line_OPTIONWARNMOUNT
%token 			OPTIONWARNMOUNT

%type <kv>		line_OPTIONEJECT
%token 			OPTIONEJECT

%type <kv>		line_COMPRESSIONLEVEL
%token 			COMPRESSIONLEVEL

%type <kv>		line_TESTVFS
%token 			TESTVFS

%type <kv>		line_VFSEXTENSION
%token 			VFSEXTENSION

%type <kv>		line_READINFORMATION
%token 			READINFORMATION

%type <kv>		line_DBCLICKFORINFO
%token 			DBCLICKFORINFO

%type <kv>		line_DISPLAYDIRSIZE
%token 			DISPLAYDIRSIZE

%type <kv>		line_DATEFORMAT
%token 			DATEFORMAT

%type <kv>		line_RECENTFILE
%token 			RECENTFILE

%type <kv>		line_RECENTFILESMAX
%token 			RECENTFILESMAX

%type <kv>		line_PREVIOUSSEARCHMODE
%token 			PREVIOUSSEARCHMODE

%type <kv>		line_SAVEONLYUSEDCATEGORIES
%token 			SAVEONLYUSEDCATEGORIES

%token <string>		STRING
%token <string>		QSTRING
%token <gbool>	        GBOOLEAN
%token <gint>		STRINGLOCALE

/* not used yet
%token <string>	EMAIL
%token 			TO
*/
%token <val>	INTEGER
%token 			EOL

%%
config_file: 	lines	{ local_config = $1; }
		;

lines:		one_line lines		{{
						if($1) {
							$$ = g_list_prepend($2, $1);
						} else {
							$$ = $2;
						}
					}}
		| one_line		{{ 
						if($1) {
							$$ = g_list_append(NULL, $1);
						} else {
							$$ = NULL;
						}
					}}
		;

one_line:	line_ADDICON EOL					{ $$ = $1; }
		| line_DEFAULTFILEICON EOL				{ $$ = $1; }
		| line_DEFAULTDIRICON EOL				{ $$ = $1; }
		| line_DEFAULTDISKICON EOL				{ $$ = $1; }
		| line_DEFAULTUNREADABLEICON EOL			{ $$ = $1; }
		| line_DEFAULTVFSICON EOL				{ $$ = $1; }
		| line_MOUNTPROGRAM EOL					{ $$ = $1; }
		| line_UMOUNTPROGRAM EOL				{ $$ = $1; }
		| line_USEAUTOMOUNT EOL					{ $$ = $1; }
		| line_EJECTPROGRAM EOL					{ $$ = $1; }
		| line_MOUNTPOINT EOL					{ $$ = $1; }
		| line_OPTIONEJECT EOL					{ $$ = $1; }
		| line_OPTIONWARNMOUNT EOL				{ $$ = $1; }
		| line_OPTIONDUS EOL					{ $$ = $1; }
		| line_COMPRESSIONLEVEL EOL				{ $$ = $1; }
		| line_TESTVFS EOL					{ $$ = $1; }
		| line_EXTENSION EOL					{ $$ = $1; }
		| line_VFSEXTENSION EOL					{ $$ = $1; }
		| line_READINFORMATION EOL				{ $$ = $1; }
		| line_DBCLICKFORINFO EOL				{ $$ = $1; }
		| line_DISPLAYDIRSIZE EOL				{ $$ = $1; }
		| line_DATEFORMAT EOL					{ $$ = $1; }
		| line_RECENTFILE EOL					{ $$ = $1; }
		| line_RECENTFILESMAX EOL				{ $$ = $1; }
		| line_PREVIOUSSEARCHMODE EOL				{ $$ = $1; }
		| line_SAVEONLYUSEDCATEGORIES EOL			{ $$ = $1; }
		| EOL							{ $$ = NULL;}
		;

line_EXTENSION:	EXTENSION GSTRING GSTRING GSTRING GSTRING GSTRING INTEGER
		{{
											KEY_VALUE *r, *r2;
											EXTENSIONS *e;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("Extension");
											r2 = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->value = r2;
											r2->key = $2;
											e = (EXTENSIONS*)g_malloc(sizeof(EXTENSIONS));
											r2->value = e;
											e->extension = g_string_new(((GString*)$2)->str);
											e->prog = $3;
											e->arg = $4;
											e->viewer = $5;
											e->viewerarg = $6;
											if(test_plugin_existence(e->prog) == FALSE)
												e->state = FALSE;
											else
												e->state = $7;
											$$ = r;
										}}
		;

line_VFSEXTENSION:	VFSEXTENSION GSTRING GSTRING GSTRING GSTRING INTEGER GSTRING INTEGER INTEGER INTEGER
		{{
											KEY_VALUE *r, *r2;
											VFS_EXTENSIONS *ve;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("VfsExtension");
											r2 = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->value = r2;
											r2->key = $2;
											ve = (VFS_EXTENSIONS*)g_malloc(sizeof(VFS_EXTENSIONS));
											r2->value = ve;
											ve->extension = g_string_new(((GString*)$2)->str);
											ve->prog = $3;
											ve->args = $4;
											ve->separator = $5;
											ve->date = $6;
											ve->date_format = $7;
											ve->size = $8;
											ve->name = $9;
											if(test_plugin_existence(ve->prog) == FALSE)
												ve->state = FALSE;
											else
												ve->state = $10;
											$$ = r;
										}}
		;

line_MOUNTPROGRAM:	MOUNTPROGRAM GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("MountProrgam");
											r->value = $2;
											$$ = r;
										}}
		;

line_UMOUNTPROGRAM:	UMOUNTPROGRAM GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("UmountProgram");
											r->value = $2;
											$$ = r;
										}}
		;

line_EJECTPROGRAM:	EJECTPROGRAM GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("EjectProgram");
											r->value = $2;
											$$ = r;
										}}
		;

line_MOUNTPOINT:	MOUNTPOINT GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("MountPoint");
											r->value = $2;
											$$ = r;
										}}
		;
line_USEAUTOMOUNT:	USEAUTOMOUNT GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("UseAutomount");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_OPTIONDUS:	        OPTIONDUS GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("OptionDus");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;
line_OPTIONWARNMOUNT:	OPTIONWARNMOUNT GBOOLEAN	{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("OptionWarnMount");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;
line_OPTIONEJECT:	OPTIONEJECT GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("OptionEject");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_COMPRESSIONLEVEL:	COMPRESSIONLEVEL INTEGER		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("CompressionLevel");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_TESTVFS:	TESTVFS INTEGER		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("TestVFS");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_READINFORMATION:	READINFORMATION GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("ReadInformation");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_DBCLICKFORINFO:	DBCLICKFORINFO GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DbClickForInfo");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;



line_DISPLAYDIRSIZE:	DISPLAYDIRSIZE GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DisplayDirSize");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;



/* line_ADDICON returns a KEY_VALUE containing a GString("AddIcon") and a glist with n elements.
 * The first element is the icon path, and the second element is the extension
 */
line_ADDICON:	ADDICON STRINGS		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("AddIcon");
											r->value = $2;
											$$ = r;
										}}
		;

line_DEFAULTFILEICON:	DEFAULTFILEICON GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DefaultFileIcon");
											r->value = $2;
											$$ = r;
										}}
		;

line_DEFAULTDIRICON:	DEFAULTDIRICON GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DefaultDirIcon");
											r->value = $2;
											$$ = r;
										}}
		;

line_DEFAULTDISKICON:	DEFAULTDISKICON GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DefaultDiskIcon");
											r->value = $2;
											$$ = r;
										}}
		;

line_DEFAULTUNREADABLEICON:	DEFAULTUNREADABLEICON GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DefaultUnreadableIcon");
											r->value = $2;
											$$ = r;
										}}
		;

line_DEFAULTVFSICON:	DEFAULTVFSICON GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DefaultVfsIcon");
											r->value = $2;
											$$ = r;
										}}
		;

line_DATEFORMAT:	DATEFORMAT GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DateFormat");
											r->value = $2;
											$$ = r;
										}}
		| DATEFORMAT STRINGLOCALE		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("DateFormatMode");
											r->value = NULL;
											$$ = r;
										}}
		;

line_RECENTFILE:	RECENTFILE GSTRING		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("RecentFile");
											r->value = $2;
											$$ = r;
										}}
		;

line_RECENTFILESMAX:	RECENTFILESMAX INTEGER		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("RecentFilesMax");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_PREVIOUSSEARCHMODE:	PREVIOUSSEARCHMODE INTEGER		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("PreviousSearchMode");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;

line_SAVEONLYUSEDCATEGORIES:	SAVEONLYUSEDCATEGORIES GBOOLEAN		{{
											KEY_VALUE *r;

											r = (KEY_VALUE*)g_malloc(sizeof(KEY_VALUE));
											r->key = g_string_new("SaveOnlyUsedCategories");
											r->value = GINT_TO_POINTER($2);
											$$ = r;
										}}
		;



STRINGS:	GSTRING				{ $$ = g_list_append(NULL, $1); }
		| GSTRING STRINGS		{ $$ = g_list_prepend($2, $1 ); }
		;

GSTRING:	STRING				{ $$ = g_string_new($1); }
		| QSTRING			{ 
							gchar *s;
							GString *g;
							s = $1;
							s++;
							g = g_string_new(s);
							$$ = g_string_truncate(g, g->len-1);
						}
		;

/* Not used yet. This is for integers.
INTEGERS: 	INTEGER				{ $$ = g_list_append(NULL, GINT_TO_POINTER($1)); }
			| INTEGER INTEGERS	{ $$ = g_list_prepend($2, GINT_TO_POINTER($1)); }
			| INTEGER TO INTEGER	{{
									gint i, b, e;
									GList *gl;
									
									if($3<$1) { b = $3; e = $1; }
									else { b = $1; e = $3; }
									gl = g_list_append(NULL, GINT_TO_POINTER(b));
									for(i=b+1; i<=e; i++) {
										gl = g_list_append(gl, GINT_TO_POINTER(i));
									}
									$$ = gl;
								}}
			| INTEGER TO INTEGER INTEGERS	{{
									gint i, b, e;
									GList *gl;
									
									if($3<$1) { b = $3; e = $1; }
									else { b = $1; e = $3; }
									gl = g_list_append($4, GINT_TO_POINTER(b));
									for(i=b+1; i<=e; i++) {
										gl = g_list_append(gl, GINT_TO_POINTER(i));
									}
									$$ = gl;
								}}
			;

*/
%%

int
yyerror (char *s)
{
  printf (_("Error parsing the config file: %s\n"), s);
  exit (0);
}

gboolean check_supermount (CONFIG * my_config)
{
  FILE *fp;
  char line[1024];
  char s1[1024];
  char s2[1024];
  char s3[1024];
  int rc;

#if (HOST_OS == linux)
  fp = fopen ("/proc/mounts", "r");
  if (fp == NULL)
    {
      ERROR_DIALOG ("Unable to open /proc/mounts\n", main_window);
    }

  while (fgets (line, sizeof (line), fp) != 0)
    {
      rc = sscanf (line, "%1023s %1023s %1023s", s1, s2, s3);
      if ((rc >= 3) && (strcmp (s2, my_config->mount_point->str) == 0)
	  && (strcmp (s3, "supermount") == 0))
	{
	  fclose (fp);
	  return TRUE;
	}
    }
  fclose (fp);
#endif
  return FALSE;
}


void
config_struct_free_1 (GList * gl)
{
  g_string_free (((KEY_VALUE *) (gl->data))->key, TRUE);
  local_config = g_list_remove_link (local_config, gl);
  g_list_free_1 (gl);
}

#define GET_TOKEN ((KEY_VALUE*)(glist_ptr1->data))->value
#define NEXT_TOKEN config_struct_free_1(glist_ptr1); glist_ptr1 = local_config
#define IS_TOKEN(my_token) (strcmp((((KEY_VALUE*)(glist_ptr1->data))->key)->str,my_token) == 0)

CONFIG *
init_config (GtkWidget * w)
{
/* Returns the config file "config_file->str" formated into the structure CONFIG */
  GString *conf_file;
  GList *glist_ptr1;
  CONFIG *my_config;
  FILE *rcfile;
  GList *gl1, *gl2;
  GString *gs;
  PIX *pix;
  GtkStyle *style;

  conf_file = g_string_new (g_get_home_dir ());
  conf_file =
    g_string_append (conf_file, USER_CONFIG_DIR "/" USER_CONFIG_FILE);
  if (!g_file_test (conf_file->str, G_FILE_TEST_ISFILE))
    {
      g_string_free (conf_file, TRUE);
      conf_file = g_string_new (DATA_DIR "/gtktalog/gtktalog.cf");
      if (!g_file_test (conf_file->str, G_FILE_TEST_ISFILE))
	{
	  ERROR_DIALOG ("Could not find config file", main_window);
	  exit (-1);
	}
    }

  local_config = NULL;

  style = gtk_widget_get_style (GTK_WIDGET (w));

  rcfile = fopen (conf_file->str, "r");
  if (rcfile)
    {

      /* Init some fields of my_config */
      my_config = (CONFIG *) g_malloc (sizeof (CONFIG));
      my_config->pix_paths = g_tree_new ((GCompareFunc) strcmp);
      my_config->icons = g_tree_new ((GCompareFunc) g_strcasecmp);
      my_config->mount = g_string_new ("/bin/mount");
      my_config->umount = g_string_new ("/bin/umount");
      my_config->eject_prog = g_string_new ("/usr/bin/eject");
      my_config->mount_point = g_string_new ("/mnt/cdrom");
      my_config->use_automount = FALSE;
      my_config->extensions = g_tree_new ((GCompareFunc) g_strcasecmp);
      my_config->vfs_extensions = g_tree_new ((GCompareFunc) g_strcasecmp);
      my_config->use_du_s = TRUE;
      my_config->warn_mount = FALSE;
      my_config->eject_disk = TRUE;
      my_config->getInformation = TRUE;
      my_config->testvfs = TEST_VFS_NO;
      my_config->compression_level = 9;
      my_config->default_file_pix = NULL;
      my_config->default_dir_pix = NULL;
      my_config->default_disk_pix = NULL;
      my_config->default_unreadable_pix = NULL;
      my_config->default_vfs_pix = NULL;
      my_config->date_format = g_strdup ("%d/%m/%Y %H:%M:%S");
      my_config->display_dir_size = TRUE;
      my_config->recent_files = NULL;
      my_config->nb_max_recent_files = 4;
      my_config->previous_search_mode = EVERYTHING_MATCHES;
      my_config->save_only_used_categories = FALSE;

      /* Parse the config file */
      yyrestart (rcfile);
      yyparse ();
      fclose (rcfile);

      /* Fills some fields of my_config with the parsed data */

      glist_ptr1 = local_config;
      while (glist_ptr1)
	{
	  if (IS_TOKEN ("AddIcon"))
	    {
	      /* Icon */
	      gl1 = GET_TOKEN;
	      pix =
		new_pix_from_filename (w, style,
				       ((GString *) (gl1->data))->str,
				       my_config);

	      gl2 = g_list_next (gl1);
	      while (gl2)
		{
		  if (pix)
		    {
		      g_tree_insert (my_config->icons,
				     g_strdup (
					       ((GString
						 *) (gl2->data))->str), pix);
		    }
		  g_string_free (gl2->data, TRUE);
		  gl2 = g_list_next (gl2);
		}

	      g_list_free (gl1);


	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DefaultFileIcon"))
	    {
	      gs = GET_TOKEN;
	      my_config->default_file_pix = new_pix_from_filename (w, style,
								   gs->str,
								   my_config);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DefaultDirIcon"))
	    {
	      gs = GET_TOKEN;
	      my_config->default_dir_pix = new_pix_from_filename (w, style,
								  gs->str,
								  my_config);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DefaultDiskIcon"))
	    {
	      gs = GET_TOKEN;
	      my_config->default_disk_pix = new_pix_from_filename (w, style,
								   gs->str,
								   my_config);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DefaultUnreadableIcon"))
	    {
	      gs = GET_TOKEN;

	      my_config->default_unreadable_pix = new_pix_from_filename (w,
									 style,
									 gs->
									 str,
									 my_config);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DefaultVfsIcon"))
	    {
	      gs = GET_TOKEN;
	      my_config->default_vfs_pix = new_pix_from_filename (w, style,
								  gs->str,
								  my_config);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("MountProgram"))
	    {
	      gs = GET_TOKEN;
	      g_string_free (my_config->mount, TRUE);
	      my_config->mount = gs;
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("UmountProgram"))
	    {
	      gs = GET_TOKEN;
	      g_string_free (my_config->umount, TRUE);
	      my_config->umount = gs;
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("EjectProgram"))
	    {
	      gs = GET_TOKEN;
	      g_string_free (my_config->eject_prog, TRUE);
	      my_config->eject_prog = gs;
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("MountPoint"))
	    {
	      gs = GET_TOKEN;
	      g_string_free (my_config->mount_point, TRUE);
	      my_config->mount_point = gs;
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("UseAutomount"))
	    {
	      gs = GET_TOKEN;
	      my_config->use_automount = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("OptionDus"))
	    {
	      gs = GET_TOKEN;
	      my_config->use_du_s = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("OptionWarnMount"))
	    {
	      gs = GET_TOKEN;
	      my_config->warn_mount = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("OptionEject"))
	    {
	      gs = GET_TOKEN;
	      my_config->eject_disk = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("CompressionLevel"))
	    {
	      gs = GET_TOKEN;
	      my_config->compression_level = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("TestVFS"))
	    {
	      gs = GET_TOKEN;
	      my_config->testvfs = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("Extension"))
	    {
	      gs = GET_TOKEN;
	      g_tree_insert (my_config->extensions,
			     ((EXTENSIONS *) ((KEY_VALUE *) gs)->value)->
			     extension->str, ((KEY_VALUE *) gs)->value);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("VfsExtension"))
	    {
	      gs = GET_TOKEN;
	      g_tree_insert (my_config->vfs_extensions,
			     ((VFS_EXTENSIONS *) ((KEY_VALUE *) gs)->value)->
			     extension->str, ((KEY_VALUE *) gs)->value);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("ReadInformation"))
	    {
	      gs = GET_TOKEN;
	      my_config->getInformation = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DbClickForInfo"))
	    {
	      gs = GET_TOKEN;
	      my_config->dbclick_opens_info = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DisplayDirSize"))
	    {
	      gs = GET_TOKEN;
	      my_config->display_dir_size = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DateFormat"))
	    {
	      gs = GET_TOKEN;
	      my_config->date_format = g_strdup (gs->str);
	      g_string_free (gs, TRUE);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("DateFormatMode"))
	    {
	      gs = GET_TOKEN;
	      my_config->date_format = NULL;
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("RecentFile"))
	    {
	      gs = GET_TOKEN;
	      my_config->recent_files =
		add_file_to_recent_files (my_config->recent_files,
					  my_config->nb_max_recent_files,
					  gs->str);
	      g_string_free (gs, TRUE);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("RecentFilesMax"))
	    {
	      gs = GET_TOKEN;
	      my_config->nb_max_recent_files = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("PreviousSearchMode"))
	    {
	      gs = GET_TOKEN;
	      my_config->previous_search_mode = GPOINTER_TO_INT (gs);
	      if (my_config->previous_search_mode >= STRING_SM_NB)
		my_config->previous_search_mode = EVERYTHING_MATCHES;
	      if (my_config->previous_search_mode < 0)
		my_config->previous_search_mode = EVERYTHING_MATCHES;
	      NEXT_TOKEN;
	    }
	  else if (IS_TOKEN ("SaveOnlyUsedCategories"))
	    {
	      gs = GET_TOKEN;
	      my_config->save_only_used_categories = GPOINTER_TO_INT (gs);
	      NEXT_TOKEN;
	    }
	  else
	    {
	      glist_ptr1 = g_list_next (glist_ptr1);
	    }
	}
      my_config->use_supermount = check_supermount (my_config);
      if (my_config->default_file_pix == NULL)
	{
	  my_config->default_file_pix = new_pix_from_filename (w, style,
							       DEFAULTFILEICONPATH,
							       my_config);
	  if (!my_config->default_file_pix)
	    exit (-1);
	}
      if (my_config->default_dir_pix == NULL)
	{
	  my_config->default_dir_pix = new_pix_from_filename (w, style,
							      DEFAULTDIRICONPATH,
							      my_config);
	  if (!my_config->default_dir_pix)
	    exit (-1);
	}
      if (my_config->default_disk_pix == NULL)
	{
	  my_config->default_disk_pix = new_pix_from_filename (w, style,
							       DEFAULTDISKICONPATH,
							       my_config);
	  if (!my_config->default_disk_pix)
	    exit (-1);
	}
      if (my_config->default_unreadable_pix == NULL)
	{
	  my_config->default_unreadable_pix = new_pix_from_filename (w, style,
								     DEFAULTUNREADABLEICONPATH,
								     my_config);
	}
      if (!my_config->default_unreadable_pix)
	exit (-1);
      if (my_config->default_vfs_pix == NULL)
	{
	  my_config->default_vfs_pix = new_pix_from_filename (w, style,
							      DEFAULTVFSICONPATH,
							      my_config);
	  if (!my_config->default_vfs_pix)
	    exit (-1);
	}
      if (my_config->eject_prog)
	{
	  if (my_config->eject_prog->len <= 0)
	    {
	      my_config->eject_disk = FALSE;
	    }
	}
      else
	{
	  my_config->eject_disk = FALSE;
	}
    }
  else
    {
      my_config = NULL;
    }
  return (my_config);
}
